function [tau_a,tau_f,s_a,s_f] = draw_tau_seas(y,sigma_dtau,sigma_eps,sigma_s)
% 
%       y: T x 1
%       sigma_dtau: T x 1
%       sigma_eps: T x 1
%       sigma_s: T x 1
%       
%       Output
%       tau_a: (T+1)x1 -- tau draw .. (t = 0 through T)
%       tau_f: Tx1 -- tau_filtered draw (t = 1 through T)
%       s_a: (T+3) x 1 -- s draws .. (t = -2, -1, 0, 1 .. through T)
      
small=1.0e-6;
big = 1.0e+6;
var_dtau = sigma_dtau.^2;
var_eps = sigma_eps.^2;
var_s = sigma_s.^2;
nobs=size(y,1);

% Set up State Vector
  % --- State Vector
  %     (1) tau(t)
  %     (2) s(t)
  %     (3) s(t-1)
  %     (4) s(t-2)
  %     (5) s(t-3)  ... add additional lag to get initial condition right
  ns = 5;      % size of state
  Q = zeros(ns,ns);
  F = zeros(ns,ns);
  F(1,1) = 1;
  F(2,2:4) = -ones(1,3);
  F(3:5,2:4) = eye(3);
  H = zeros(ns,1);
  H(1,1) = 1;
  H(2,1) = 1;
  
  
  % Initial conditions
  X1_init = zeros(ns,1);
  P1_init = zeros(ns,ns);
  P1_init(1,1) = big;
  % Impose Constraint that s(0)+s(-1)+s(-2)+s(-3) = 0 (so mean is in tau compnents)
  A = eye(4)-ones(4,4)/4;
  P1_init(2:5,2:5) = big*A*A';  % Vague prior for seasonal pattern (but sum to zero)
  
  X1 = X1_init;
  P1 = P1_init;
  X1t=zeros(ns,nobs+1);
  P1t=zeros(ns,ns,nobs+1);
  X2t=zeros(ns,nobs+1);
  P2t=zeros(ns,ns,nobs+1);
  X1t(:,1)=X1; 
  P1t(:,:,1)=P1;
  X_Draw_Filt = NaN*zeros(nobs+1,ns);  % Draw from filtered .. these are marginal draws, not joint (same as ucsv) ;
  
  % Check for missing values
  if sum(sum(isnan(y))) == 0;
   for t = 1:nobs;
    H_t = H;
    Q_t = Q;
    Q_t(1,1) = var_dtau(t);
    Q_t(2,2) = var_s(t);
    R_t = var_eps(t);
    yt = y(t);
    [X1,P1,X2,P2] = kfilt(yt,X1,P1,H_t,F,R_t,Q_t);
    X1t(:,t+1)=X1;
    P1t(:,:,t+1)=P1;
    X2t(:,t+1)=X2;
    P2t(:,:,t+1)=P2;
    chol_P1 = chol(P1);
    X = X1+chol_P1'*randn(ns,1);
    X_Draw_Filt(t+1,:)=X';
   end;
  
  else;
   for t = 1:nobs;
    Q_t = Q;
    Q_t(1,1) = var_dtau(t);
    Q_t(2,2) = var_s(t);
    if isnan(y(t)) == 1;
      X2 = F*X1;
      P2 = F*P1*F' + Q_t;
      X1 = X2;
      P1 = P2;
    else;
      H_t = H;
      R_t = var_eps(t);
      yt = y(t);
      [X1,P1,X2,P2] = kfilt(yt,X1,P1,H_t,F,R_t,Q_t);
    end;
    X1t(:,t+1)=X1;
    P1t(:,:,t+1)=P1;
    X2t(:,t+1)=X2;
    P2t(:,:,t+1)=P2;
    chol_P1 = chol(P1);
    X = X1+chol_P1'*randn(ns,1);
    X_Draw_Filt(t+1,:)=X';
   end;
 
 end; 
 
 % Draw From State
   X_Draw = NaN*zeros(nobs+1,ns);
   % Initial Draw
   P3 = P1;
   X3 = X1;
   P3s = chol(P3)';
   X = X3+P3s*randn(ns,1);
   X_Draw(nobs+1,:)=X';
   for t = nobs:-1:1;
      X1 = X1t(:,t);
      X2 = X2t(:,t+1);
      P1 = P1t(:,:,t);
      P2 = P2t(:,:,t+1);
      F_P1 = F*P1;
      [AS,rcond]=linsolve(P2,F_P1);
      if rcond < 1.0e-12;;
           P2i=pinv(P2);
           AS = P2i*F_P1;
      end;
      P3=P1-AS'*F_P1;   % Note .. this is NOT Kalman Smoother .. here we are working back recursively
      P3 = 0.5*(P3+P3');
      X3=X1+AS'*(X-X2);
      X = X3;
      P3s = chol(P3+small*eye(ns))';   
      X = X + P3s*randn(ns,1);           
      X_Draw(t,:)=X';
   end;
   tau_a = X_Draw(:,1);
   s_a = [X_Draw(1,4);X_Draw(1,3);X_Draw(:,2)];
   tau_f = X_Draw_Filt(2:end,1);
   s_f = X_Draw_Filt(2:end,2);
   
end